aglm <- function(formula, data, sp = ~first, family=binomial,
                 brlr=TRUE, bootstrap=TRUE, nboot=99, 
                 mctest=FALSE, nmc=99, bor=1, lag=-1, times=30) {

  aglm.env <- new.env()

  id.resp <- which(is.element(dimnames(data@data@value)[[4]],
                              strsplit(as.character(formula)[[2]], ' ')[[1]]))
  id.covar <- which(is.element(dimnames(data@data@value)[[4]],
                               strsplit(as.character(formula)[[3]], ' ')[[1]]))
  xcov <- NULL
  if (length(id.covar)>0) {
    for (i in id.covar)
      xcov <- cbind(xcov,
                    as.numeric(.extract.border(data@data@value[,,,i],
                                               bor=bor)))
    colnames(xcov) <- dimnames(data@data@value)[[4]][id.covar]
  }
  cform <- as.character(formula)
  csp <- as.character(sp)
  formula <- as.formula(paste(cform[2], cform[1],
                              paste(cform[3], csp[2], sep="+")))

  dddd <- dim(data@data@value)
  if (dddd[3]==1) {
    fit <- function() {
      y <- matrix(get("y", envir=aglm.env),
                  dddd[1], dddd[2])
      y[is.na(data@data@value[,,,id.resp])] <- NA
      if (is.null(xcov))
        dataf <- data.frame(as.numeric(.extract.border(y, bor=bor)),
                            sumNeighbor3(sp, y, lag=lag, bor=bor))
      else
        dataf <- data.frame(as.numeric(.extract.border(y, bor=bor)),
                            xcov, sumNeighbor3(sp, y, lag=lag, bor=bor))
      names(dataf)[1] <- dimnames(data@data@value)[[4]][id.resp]
      if (brlr)
        return(brlr(formula, data=dataf))
      else
        return(glm(formula, data=dataf, family=family))
    }
    y <- data@data@value[,,,id.resp]
    y[is.na(y)] <- 0
    assign("y", as.numeric(y),
           envir=aglm.env)
    n <- prod(dddd[1:2]-c(2,2))
   
    res <- list(naive=fit())
    if (bootstrap) {
      hbeta <- coef(res$naive)
      nmb <- names(hbeta)
      defterms <- c("(Intercept)", "R", "C", "first",
                    "dA", "dB", "D", "second")
      cbeta <- rep(0, 8)
      cbeta[is.element(defterms, nmb)] <- hbeta[is.element(nmb, defterms)]
      cbeta <- c(cbeta, hbeta[!is.element(nmb, defterms)])
      id.m <- matrix(as.integer(1:(dddd[1]*dddd[2])), dddd[1], dddd[2])
      aord <- as.integer(sapply(2:(dddd[1]-1), function(i)
                                sapply(2:(dddd[2]-1), function(j) 
                                       id.m[i,j])))-1
      no.na <- data@data@value
      no.na[is.na(data@data@value)] <- 0
      res$boot <- t(sapply(1:nboot, function(i) {
        assign("y", .C('rautobin',
                       as.integer(get("y", envir=aglm.env)), 
                       as.double(no.na[,,,id.covar]),
                       as.double(cbeta), as.integer(dddd[1]),
                       as.integer(dddd[2]), as.integer(length(id.covar)),
                       as.integer(sample(aord)), DUP=FALSE,
                       PACKAGE="strLattice")[[1]],
               envir=aglm.env)
        coef(fit())
      }))
    }
    if (mctest) {
      res$mc <- t(sapply(1:nmc, function(i) {
        assign("y", sample(get("y", envir=aglm.env)), 
               envir=aglm.env)
        coef(fit())
      }))
    }
  }
  else {
    nloglikbin <- function(beta) {
      xb <- drop(X%*%beta)
      pb <- 1/(1+dt*exp(-xb))
      ### verificar 'dt'
      obj <- -sum(dbinom(Y, 1, pb, log=TRUE))
      if (brlr)
        obj <- obj -0.5*determinant(crossprod(X*(pb*(1-pb)),X))$modulus
      return(obj)
    }
    if (lag==0)
      Y <- as.numeric(.extract.border(data@data@value[,,,id.resp], bor=bor))
    else
      Y <- as.numeric(.extract.border(data@data@value
                                      [,,-(1:(-lag)), id.resp], bor=bor))
    n <- length(Y)
    X <- matrix(1,n,1)
    if (!is.null(xcov)) {
      if (lag==0)
        X <- cbind(X, xcov)
      else 
        X <- cbind(X, xcov[setdiff(1:nrow(xcov),
                                   1:(prod(dddd[1:2]-2*bor)*-lag)),])
    }
##    if (length(id.covar)>0)
  ##    X <- cbind(X, sapply(id.covar, function(i) {
    ##    as.numeric(.extract.border(data@data@value
      ##                             [,,-(1:(-lag)), i], bor=bor))}))
    colnames(X) <- c("(Intercept)", dimnames(data@data@value)[[4]][id.covar])
    X <- cbind(X, sumNeighbor3(sp, data@data@value[,,,id.resp],
                               lag=lag, bor=bor))
    if (lag==0)
      dt <- 1
    else 
      dt <- rep(as.numeric(diff(data@dates))/times, each=prod(dddd[1:2]-2*bor))
    datf <- data.frame(Y,X[,-1])
    colnames(datf) <- c(as.character(formula)[2], colnames(X)[-1])
    res <- list(naive=glm(formula, family=family,
                  data=datf))
    rm(datf)
    opt <- optim(coef(res$naive), nloglikbin,
                 method="BFGS", hessian=TRUE)
    res$coefficients <- opt$par
    res$linear.predictors <- X%*%opt$par
    res$fitted.values <- family()$linkinv(res$linear.predictors)
    res$residuals <- Y-res$fitted.values
    res$logLik <- sum(dbinom(Y, 1, res$fitted.values, log=TRUE))
    res$npars <- ncol(X)
    res$df.null <- length(Y)
    res$df.residual <- res$df.null-res$npars
    res$deviance <- -2*res$logLik
    res$null.deviance <-
      -2*sum(dbinom(Y, 1, fitted(glm(Y~1,binomial)), log=TRUE))
    res$optim.res <- opt
  }
  class(res) <- "aglm"
  return(res)
}

